'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.errorCatchHandler = exports.verifyOutput = exports.regexResult = exports.repoHasLfs = exports.repoHasLfsObjectBin = exports.repoHasLfsFilters = exports.loadGitattributeFiltersFromRepo = exports.getGitattributesPathFromRepo = undefined;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _ramda = require('ramda');

var _ramda2 = _interopRequireDefault(_ramda);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getGitattributesPathFromRepo = exports.getGitattributesPathFromRepo = function getGitattributesPathFromRepo(repo) {
  return _path2.default.join(repo.workdir(), '.gitattributes');
};

var loadGitattributeFiltersFromRepo = exports.loadGitattributeFiltersFromRepo = function loadGitattributeFiltersFromRepo(repo) {
  var gitattrPath = getGitattributesPathFromRepo(repo);
  return _fsExtra2.default.pathExists(gitattrPath).then(function (exists) {
    if (!exists) {
      return '';
    }

    return _fsExtra2.default.readFile(gitattrPath, 'utf8');
  }).then(function (fileContents) {
    var attributes = fileContents.split('\n');
    var lfsFilters = _ramda2.default.reduce(function (acc, line) {
      if (!_ramda2.default.contains(_constants.LFS_ATTRIBUTE, line)) {
        return acc;
      }

      var start = line.indexOf(_constants.LFS_ATTRIBUTE);
      var filter = line.substring(0, start);
      acc.push(filter.trim());

      return acc;
    }, [], attributes);

    return lfsFilters;
  });
};

var repoHasLfsFilters = exports.repoHasLfsFilters = function repoHasLfsFilters(repo) {
  return loadGitattributeFiltersFromRepo(repo).then(function (filters) {
    return filters.length > 0;
  }).catch(function () {
    return false;
  });
};

var repoHasLfsObjectBin = exports.repoHasLfsObjectBin = function repoHasLfsObjectBin(repo) {
  return _fsExtra2.default.pathExists(_path2.default.join(repo.workdir(), '.git', 'lfs'));
};

var repoHasLfs = exports.repoHasLfs = function repoHasLfs(repo) {
  return repoHasLfsFilters(repo).then(function (hasFilters) {
    return hasFilters || repoHasLfsObjectBin(repo);
  });
};

var regexResult = exports.regexResult = function regexResult(input, regularExpression) {
  return input.match(regularExpression);
};

var verifyOutput = exports.verifyOutput = function verifyOutput(stats, raw) {
  // Check to see if it was a permissions error
  var wasPermissionDenied = raw.trim().toLowerCase().match(_constants.regex.PERMISSION_DENIED);
  if (wasPermissionDenied) {
    var e = new Error(wasPermissionDenied[0]);
    e.errno = _constants.BAD_CORE_RESPONSE;
    throw e;
  }

  // We need to handle this manually because LFS isn't returning stderr
  var props = _ramda2.default.values(stats);
  var allErrored = _ramda2.default.pipe(_ramda2.default.filter(_ramda2.default.equals(_constants.BAD_REGEX_PARSE_RESULT)), _ramda2.default.sum, _ramda2.default.equals(props.length))(props);

  // We have all errors
  if (allErrored) {
    var _e = new Error(raw);
    _e.errno = _constants.BAD_CORE_RESPONSE;
    throw _e;
  }
};

var errorCatchHandler = exports.errorCatchHandler = function errorCatchHandler(response) {
  return function (err) {
    // This is a manually detected error we get from LFS
    if (err.errno === _constants.BAD_CORE_RESPONSE) {
      response.stderr = response.raw;
      response.errno = _constants.BAD_CORE_RESPONSE;
      response.success = false;
      return response;
    }

    throw err;
  };
};