'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.dependencyCheck = exports.isAtleastLfsVersion = exports.isAtleastGitVersion = exports.parseVersion = undefined;

var _ramda = require('ramda');

var _ramda2 = _interopRequireDefault(_ramda);

var _generateResponse = require('./generateResponse');

var _generateResponse2 = _interopRequireDefault(_generateResponse);

var _lfsCommands = require('../commands/lfsCommands');

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @function normalizeVersion
 * @param  Array<string> versionArray array of version number eg: ['1', '8', '3'] => 1.8.3
 * @return Number normalized version number
 */
var normalizeVersion = function normalizeVersion(versionArray) {
  if (!versionArray || versionArray.length === 0) {
    return _constants.BAD_VERSION;
  }
  return _ramda2.default.join('.', versionArray);
};

var parseVersion = exports.parseVersion = function parseVersion(input, regex) {
  if (!input) {
    return _constants.BAD_VERSION;
  }

  var matches = input.match(regex);
  if (!matches || _ramda2.default.isEmpty(matches)) {
    return _constants.BAD_VERSION;
  }

  var numericVersionNumbers = _ramda2.default.filter(function (match) {
    return !isNaN(match);
  }, matches);
  if (numericVersionNumbers.length > 0) {
    return normalizeVersion(numericVersionNumbers);
  }
  return matches[1];
};

var isAtleastGitVersion = exports.isAtleastGitVersion = function isAtleastGitVersion(gitInput) {
  return parseVersion(gitInput, _constants.regex.GIT) >= _constants.minimumVersions.GIT;
};

var isAtleastLfsVersion = exports.isAtleastLfsVersion = function isAtleastLfsVersion(lfsInput) {
  return parseVersion(lfsInput, _constants.regex.LFS) >= _constants.minimumVersions.LFS;
};

var setLfsFailed = function setLfsFailed(response) {
  response.success = false;
  response.errno = _constants.BAD_VERSION;
  response.lfs_meets_version = false;
  response.lfs_exists = false;
  response.lfs_raw = null;
  response.lfs_version = null;
};

var setGitFailed = function setGitFailed(response) {
  response.success = false;
  response.errno = _constants.BAD_VERSION;
  response.git_meets_version = false;
  response.git_exists = false;
  response.git_raw = null;
  response.git_version = null;
};

var dependencyCheck = exports.dependencyCheck = function dependencyCheck() {
  var response = (0, _generateResponse2.default)();
  return _lfsCommands.core.git('--version').then(function (_ref) {
    var stdout = _ref.stdout,
        stderr = _ref.stderr;

    if (stderr) {
      setGitFailed(response);
    } else {
      response.git_version = parseVersion(stdout, _constants.regex.GIT);
      response.git_meets_version = isAtleastGitVersion(stdout);
      response.git_exists = response.git_version !== _constants.BAD_VERSION;
      response.git_raw = stdout;
    }
  }).catch(function () {
    setGitFailed(response);
  }).then(function () {
    return _lfsCommands.core.git('lfs version');
  }).then(function (_ref2) {
    var stdout = _ref2.stdout,
        stderr = _ref2.stderr;

    if (stderr) {
      setLfsFailed(response);
    } else {
      response.lfs_version = parseVersion(stdout, _constants.regex.LFS);
      response.lfs_meets_version = isAtleastLfsVersion(stdout);
      response.lfs_exists = response.lfs_version !== _constants.BAD_VERSION;
      response.lfs_raw = stdout;
    }

    return response;
  }).catch(function () {
    setLfsFailed(response);
    return response;
  });
};